<?php

namespace TDevAgency\Cms\Controllers\Admin;

use Illuminate\Http\RedirectResponse;
use Illuminate\Http\Request;
use Illuminate\Support\Facades\DB;
use Illuminate\Support\Facades\Hash;
use Illuminate\Support\Facades\Redirect;
use Illuminate\Support\Facades\Response;
use Illuminate\Support\Facades\Validator;
use Illuminate\Support\Facades\View;
use Illuminate\Validation\ValidationException;
use TDevAgency\Cms\Controllers\Controller;
use TDevAgency\Cms\Models\BaseModels\AdminUser;
use TDevAgency\Cms\Models\BaseModels\AdminUserRole;
use Throwable;

class UsersController extends Controller
{
    /**
     * @return \Illuminate\Contracts\View\View
     */
    public function index(): \Illuminate\Contracts\View\View
    {
        $adminUser = AdminUser::sortable()->with('roles')->orderBy('id')->paginate(15);
        $roles = $this->roles($adminUser);

        return View::make('cms::admin.screens.admin-users.index')->with('users', $adminUser)->with('roles', $roles);
    }

    /**
     * @param $adminUser
     * @return array
     */
    private function roles($adminUser): array
    {
        $roles = [];
        foreach ($adminUser as $user) {
            $user_roles = [];
            foreach ($user->roles as $role) {
                $user_roles[] = $role->admin_role;
            }
            $roles[$user->id] = implode(", ", $user_roles);
        }
        return $roles;
    }

    /**
     * @return \Illuminate\Contracts\View\View
     */
    public function create(): \Illuminate\Contracts\View\View
    {
        return View::make('cms::admin.screens.admin-users.create');
    }

    /**
     * @param Request $request
     * @return RedirectResponse
     * @throws Throwable
     */
    public function store(Request $request): RedirectResponse
    {
        $validator = Validator::make($request->all(), [
            'name' => 'required',
            'email' => 'required',
            'password' => 'required|min:8',
        ]);

        if ($validator->fails()) {
            return Redirect::back()->withErrors($validator->errors())->withInput();
        }

        $user = new AdminUser();
        try {
            DB::transaction(function () use ($request, &$user, &$role) {
                foreach ($user->getFillable() as $field) {
                    if ($request->has($field)) {
                        if ($field === 'password') {
                            $user->password = Hash::make($request->get($field));
                        } else {
                            $user->$field = $request->get($field);
                        }
                    }
                }
                $user->save();

                if ($request->has('roles')) {
                    foreach ($request->get('roles') as $admin_role) {
                        $role = new AdminUserRole();
                        $role->admin_role = $admin_role;
                        $role->role_status = $request->get('is_active');
                        $role->admin_user_id = $user->id;
                        $role->save();
                    }
                }
            });
            return Redirect::route('cms.admin-users.index')->with('toast', [
                'title' => trans('cms::admin.toast.created'),
                'status' => 'success'
            ]);
        } catch (Throwable $e) {
            return Redirect::back()->withInput()->with('toast', [
                'title' => trans('cms::admin.toast.not_created'),
                'status' => 'danger'
            ]);
        }
    }

    /**
     * @param $id
     */
    public function show($id)
    {
        //
    }

    /**
     * @param $id
     * @return \Illuminate\Contracts\View\View|void
     */
    public function edit($id)
    {
        if (isset($id)) {
            $adminUser = AdminUser::where('id', $id)->with('roles')->firstOrFail();
            if (isset($adminUser)) {
                $user_roles = [];
                foreach ($adminUser['roles'] as $role) {
                    $user_roles[$role->admin_role] = $role->admin_role;
                }
                return View::make('cms::admin.screens.admin-users.edit')->with('adminUser', $adminUser)->with(
                    'roles',
                    $user_roles
                );
            }
            return Response::make(null, \Symfony\Component\HttpFoundation\Response::HTTP_NOT_FOUND);
        }
        return Response::make(null, \Symfony\Component\HttpFoundation\Response::HTTP_NOT_FOUND);
    }

    /**
     * @param Request $request
     * @param $id
     * @return RedirectResponse
     * @throws ValidationException
     * @throws Throwable
     */
    public function update(Request $request, $id): RedirectResponse
    {
        $this->validate($request, [
            'name' => 'required',
            'email' => 'required',
        ]);

        $adminUser = AdminUser::find($id);
        DB::transaction(function () use ($request, $adminUser) {
            foreach ($adminUser->getFillable() as $field) {
                if ($request->has($field)) {
                    if ($field === 'password') {
                        if (
                            !(Hash::check(trim($request->get($field)), $adminUser->password)) && (trim(
                                $request->get($field)
                            )) !== ''
                        ) {
                            $adminUser->password = Hash::make($request->get($field));
                        }
                    } else {
                        $adminUser->$field = $request->get($field);
                    }
                }
            }
            $adminUser->update();
        });

        try {
            if ($request->has('roles')) {
                AdminUserRole::where('admin_user_id', $id)->whereNotIn('admin_role', $request->get('roles'))->delete(
                    ['role_status' => 0]
                );
                DB::transaction(function () use ($request, $id) {
                    foreach ($request->get('roles') as $key => $value) {
                        AdminUserRole::updateOrCreate(
                            [
                                'admin_user_id' => $id,
                                'admin_role' => $value,
                            ],
                            [
                                'admin_role' => $value,
                                'role_status' => 1,
                            ]
                        );
                    }
                });
            }
            return Redirect::route('cms.admin-users.index')->with('toast', [
                'title' => trans('cms::admin.toast.updated'),
                'status' => 'success'
            ]);
        } catch (Throwable $e) {
            return Redirect::back()->withInput()->with('toast', [
                'title' => trans('cms::admin.toast.not_updated'),
                'status' => 'danger'
            ]);
        }
    }

    /**
     * @param $id
     * @return RedirectResponse
     * @throws Throwable
     */
    public function destroy($id): RedirectResponse
    {
        if (isset($id)) {
            try {
                DB::transaction(function () use ($id) {
                    AdminUserRole::where('admin_user_id', $id)->delete();
                    AdminUser::where('id', $id)->delete();
                });
            } catch (Throwable $exception) {
                return Redirect::back()->with('toast', [
                    'title' => trans('cms::admin.toast.not_deleted'),
                    'status' => 'danger'
                ]);
            }
        }
        return Redirect::back()->with('toast', [
            'title' => trans('cms::admin.toast.deleted'),
            'status' => 'success'
        ]);
    }
}
